/**
 * Bundle of: vee-element
 * Generated: 2019-12-17
 * Version: 1.0.0-alpha.5
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('object-get')) :
  typeof define === 'function' && define.amd ? define(['exports', 'object-get'], factory) :
  (factory((global.VeeElement = {}),global.get));
}(this, (function (exports,get) { 'use strict';

  get = get && get.hasOwnProperty('default') ? get['default'] : get;

  function noop () { }

  function isEmpty (value) {
    return value === void 0 ||
      value === null ||
      value === ''
  }

  var VeeDriver = function VeeDriver (validator) {
    this.validator = validator;
  };

  VeeDriver.prototype.validate = function validate (trigger, callback, driver) {
      var this$1 = this;
      if ( callback === void 0 ) callback = noop;
      if ( driver === void 0 ) driver = null;


    // this.validateDisabled = false
    if (trigger === 'blur' && !isEmpty(this.fieldValue)) {
      return
    }

    var rules = this.getRules();
    if (!rules) {
      callback();
    }

    var verifyOptions = {
      name: this.label,
    };

    // this.validateState = 'validating'
    var validator = driver.validator;
    validator
      .verify(this.fieldValue, rules, verifyOptions)
      .then(function (ref) {
          var valid = ref.valid;
          var errors = ref.errors;

        // variables
        var prop = this$1.prop;
        var error = errors[0];

        // generate error messages
        var errorField = error || '';
        var errorForm = error || null;
        var invalidFields = {};
        if (!valid) {
          invalidFields[prop] = [{
            field: prop,
            label: this$1.label,
            message: errorForm
          }];
        }

        // update
        this$1.validateState = valid ? 'success' : 'error';
        this$1.validateMessage = errorField;

        // respond
        callback(errorForm, invalidFields);
        this$1.elForm && this$1.elForm.$emit('validate', prop, !errors, errorForm);
      });
  };

  VeeDriver.prototype.getRules = function getRules () {
    var rules = this.rules
      ? this.rules
      : this.prop
        ? get(this.form.rules, this.prop) || ''
        : '';

    if (typeof rules === 'object' && !Array.isArray(rules)) {
      Object.defineProperty(rules, 'length', { value: 1 });
    }

    return rules
  };

  VeeDriver.prototype.getFilteredRule = function getFilteredRule () {
    return this.getRules()
  };

  VeeDriver.prototype.isRequired = function isRequired () {
    var rules = this.getRules();
    return typeof rules === 'string'
      ? /\brequired\b/.test(rules)
      : !!get(rules, 'required')
  };

  var Async = function Async (options) {
    this.validate = options.methods.validate;
    this.getRules = options.methods.getRules;
    this.getFilteredRule = options.methods.getFilteredRule;
    this.isRequired = options.computed.isRequired;
  };

  function main (Vue, validator, asDefault) {
    if ( asDefault === void 0 ) asDefault = true;


    // form
    Vue.options.components.ElForm.options.props.driver = String;

    // form item
    var ElFormItem = Vue.options.components.ElFormItem;

    // drivers
    var defaultDriver = asDefault
      ? 'vee'
      : 'async';
    var drivers = {
      async: new Async(ElFormItem.options),
      vee: new VeeDriver(validator),
    };

    // props
    Vue.options.components.ElFormItem = ElFormItem.extend({
      props: {
        rules: [Object, String, Array],
        driver: String
      },

      computed: {
        isRequired: function isRequired () {
          return this.selectedDriver.isRequired.apply(this)
        },

        selectedDriver: function selectedDriver () {
          return drivers[this.form.driver || defaultDriver]
        }
      },

      methods: {
        validate: function validate (trigger, callback) {
          var driver = this.selectedDriver;
          return driver.validate.apply(this, [trigger, callback, driver]) // pass driver as 3rd argument to get around binding
        },

        getRules: function getRules () {
          return this.selectedDriver.getRules.apply(this)
        },

        getFilteredRule: function getFilteredRule (trigger) {
          return this.selectedDriver.getFilteredRule.apply(this, [trigger])
        }
      }
    });
  }

  exports.default = main;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
